require 'spec_helper'

describe Barcoder::Bucket do

  it "loads all buckets" do
    buckets = Barcoder::Bucket.all
    buckets.size.should be > 0

    buckets.first.persisted.should be_true
  end

  context "an existing bucket" do
    before(:each) do
      @bucket = Barcoder::Bucket.all.first
    end

    context "#save" do
      it "updates the existing bucket" do
        token = @bucket.token # this is how we can ensure we get the same bucket
        @bucket.name = 'updated name'

        @bucket.save.should be_true

        @bucket.name.should eq 'updated name'
        @bucket.token.should eq token
        @bucket.persisted.should be_true
      end
    end

    it "#decode_from_file" do
      Barcoder::PDF.should_receive(:decode_from_file).
        with(@bucket, '/test/file')
      @bucket.decode_from_file('/test/file')
    end

    it "#decode_from_string" do
      Barcoder::PDF.should_receive(:decode_from_string).
        with(@bucket, 'content')
      @bucket.decode_from_string('content')
    end
  end

  context "a new bucket" do
    before(:each) do
      @bucket = Barcoder::Bucket.new(name: 'new bucket')
    end

    it "should not be persisted" do
      @bucket.persisted.should be_false
    end

    it "should be valid with a name" do
      @bucket.valid?.should be_true
      @bucket.name = nil
      @bucket.valid?.should be_false
    end

    it "should not allow decoding without a token" do
      @bucket.decode_from_file('test').should be_false
      @bucket.decode_from_string('test').should be_false
    end

    context "#save" do
      it "returns false when invalid" do
        @bucket.stub(:valid?).and_return(false)
        @bucket.save.should be_false
      end

      it "persists and updates with new data from server when valid" do
        @bucket.save.should be_true

        @bucket.name.should eq 'new bucket'
        @bucket.token.should_not be_nil
        @bucket.persisted.should be_true
      end
    end
  end

  context "#unmatched_pages" do
    before(:each) do
      @bucket = Barcoder::Bucket.all.first
      @bucket.unmatched_pages
    end

    it "caches unmatched_pages" do
      @bucket.should_not_receive(:unmatched_pages_request)
      @bucket.unmatched_pages
    end

    it "breaks the cache when requested" do
      @bucket.should_receive(:unmatched_pages_request).
        and_return(mock(run: true))
      @bucket.unmatched_pages(false)
    end
  end

  context "#recent_imports" do
    before(:each) do
      @bucket = Barcoder::Bucket.all.first
      @bucket.recent_imports(Time.now)
    end

    it "caches recent_imports" do
      @bucket.should_not_receive(:recent_imports_request)
      @bucket.recent_imports(Time.now)
    end

    it "breaks the cache when requested" do
      @bucket.should_receive(:recent_imports).
        and_return(mock(run: true))
      @bucket.recent_imports(Time.now, false)
    end
  end

  context "#partial_imports" do
    before(:each) do
      @bucket = Barcoder::Bucket.all.first
      @bucket.partial_imports
    end

    it "caches partial_imports" do
      @bucket.should_not_receive(:partial_imports_request)
      @bucket.partial_imports
    end

    it "breaks the cache when requested" do
      @bucket.should_receive(:partial_imports_request).
        and_return(mock(run: true))
      @bucket.partial_imports(false)
    end
  end

  context "#queue_" do
    before(:each) do
      @bucket = Barcoder::Bucket.all.first
      @bucket.queue_unmatched_pages
    end

    it "should not run the request yet" do
      @bucket.instance_eval("@unmatched_pages").should be_nil
    end

    it "should fetch the unmatched_pages when queue run" do
      Barcoder::Config.hydra.run
      @bucket.instance_eval("@unmatched_pages").should be_an Array
    end
  end
end
