require 'spec_helper'

describe Barcoder::PDF do

  before(:each) do
    @bucket = Barcoder::Bucket.new(token: '123testbucket')
    @pdf = Barcoder::PDF.new pdf_content: 'content', token: 'token'
  end

  it "should be invalid without pdf_content" do
    @pdf.pdf_content = ''
    @pdf.valid?.should be_false
  end

  it "should be invalid without token" do
    @pdf.token = ''
    @pdf.valid?.should_not be_true
  end

  it "should allow bucket to provide the token" do
    @pdf.token = nil
    @pdf.bucket = @bucket
    @pdf.token.should eq '123testbucket'
  end

  it "::new_from_file" do
    unbarcoded = 'spec/assets/unbarcoded.pdf'
    pdf = Barcoder::PDF.new_from_file(unbarcoded)

    # Original file content
    f = File.open(unbarcoded, 'r', encoding: 'ascii-8bit')
    content = f.read
    f.close

    pdf.pdf_content.should eq content
  end

  context "#barcode!" do
    before(:each) do
      @pdf = Barcoder::PDF.new_from_file('spec/assets/unbarcoded.pdf')
      @orig_pdf_content = @pdf.pdf_content
      @pdf.token = '123testbucket'
      @pdf.barcode!
    end
    it "updates with the barcoded version" do
      @pdf.pdf_content.should_not eq @orig_pdf_content
      # If we match the string PDF then we know we have properly performed
      # decode64
      @pdf.pdf_content.should =~ /PDF/
      @pdf.messages.size.should eq 2
      @pdf.id.present?.should be_true
    end

    context "::get" do
      before(:each) do
        @pdf = Barcoder::PDF.get(@bucket, @pdf.id)
      end
      it "fetches PDF with Pages" do
        @pdf.pages.size.should eq 2
        @pdf.pages.first.should be_a Barcoder::Page

        @pdf.total_pages_count.should eq 2
        @pdf.missing_pages_count.should eq 2
      end
    end
  end

  context "#queue_barcode" do
    before(:each) do
      opts = {token: '123testbucket'}

      @pdf1 = Barcoder::PDF.new_from_file('spec/assets/unbarcoded-1.pdf', opts)
      @pdf1.queue_barcode
      @pdf2 = Barcoder::PDF.new_from_file('spec/assets/unbarcoded-2.pdf', opts)
      @pdf2.queue_barcode
    end

    it "should not run the requests yet" do
      @pdf.id.should be_nil
    end

    it "updates with the barcoded version when we run the queued requests" do
      Barcoder::Config.hydra.run
      # If we match the string PDF then we know we have properly performed
      # decode64
      @pdf1.pdf_content.should =~ /PDF/
      @pdf1.messages.size.should eq 1
      @pdf1.id.present?.should be_true

      @pdf2.pdf_content.should =~ /PDF/
      @pdf2.messages.size.should eq 1
      @pdf2.id.present?.should be_true
    end
  end

  context "decoding" do
    it "#decode_from_file" do
      barcoded = 'spec/assets/barcoded-1.pdf'
      Barcoder::PDF.should_receive(:decode_from_string).with(@bucket, File.read(barcoded))
      Barcoder::PDF.decode_from_file(@bucket, barcoded)
    end

    it "#decode_from_string" do
      barcoded = 'spec/assets/barcoded-1.pdf'
      res = Barcoder::PDF.decode_from_string(@bucket, File.read(barcoded))
      res.should be_true
    end

    it "returns false when not a valid PDF" do
      res = Barcoder::PDF.decode_from_string(@bucket, 'invalid PDF content')
      res.should be_false
    end
  end

end
